@@ -1,7 +1,7 @@
 import numpy as np
 from itertools import chain
-from PyQt4 import QtGui
-from PyQt4.QtCore import Qt
+from AnyQt import QtWidgets
+from AnyQt.QtCore import Qt
 
 from Orange.data import Table, Domain, ContinuousVariable, StringVariable
 from Orange.classification.logistic_regression import LogisticRegressionLearner
@@ -77,7 +77,7 @@ class OWLogisticRegression(OWBaseLearner):
     def update_model(self):
         super().update_model()
         coef_table = None
-        if self.valid_data:
+        if self.model is not None:
             coef_table = create_coef_table(self.model)
         self.send("Coefficients", coef_table)
 
@@ -105,7 +105,7 @@ def create_coef_table(classifier):
 
 
 if __name__ == "__main__":
-    app = QtGui.QApplication([])
+    app = QtWidgets.QApplication([])
     w = OWLogisticRegression()
     w.set_data(Table("zoo"))
     w.show()
